@@ -29,24 +29,52 @@
 #include "nm-device.h"
 #include "nm-ip4-config.h"
 #include "nm-setting-ip4-config.h"
+#include "nm-ip6-config.h"
+#include "nm-setting-ip6-config.h"
 #include "nm-connection.h"
 
 gboolean nm_ethernet_address_is_valid (const struct ether_addr *test_addr);
 
 int nm_spawn_process (const char *args);
 
-void nm_print_device_capabilities (NMDevice *dev);
-
-char *nm_utils_hexstr2bin (const char *hex, size_t len);
-
 char *nm_ether_ntop (const struct ether_addr *mac);
 
 void nm_utils_merge_ip4_config (NMIP4Config *ip4_config, NMSettingIP4Config *setting);
+void nm_utils_merge_ip6_config (NMIP6Config *ip6_config, NMSettingIP6Config *setting);
 
 void nm_utils_call_dispatcher (const char *action,
                                NMConnection *connection,
                                NMDevice *device,
                                const char *vpn_iface);
 
-#endif
+gboolean nm_match_spec_hwaddr (const GSList *specs, const char *hwaddr);
 
+
+GHashTable *value_hash_create          (void);
+void        value_hash_add             (GHashTable *hash,
+										const char *key,
+										GValue *value);
+
+void        value_hash_add_str         (GHashTable *hash,
+										const char *key,
+										const char *str);
+
+void        value_hash_add_object_path (GHashTable *hash,
+										const char *key,
+										const char *op);
+
+void        value_hash_add_uint        (GHashTable *hash,
+										const char *key,
+										guint32 val);
+
+void        value_hash_add_bool        (GHashTable *hash,
+					                    const char *key,
+					                    gboolean val);
+
+gboolean nm_utils_do_sysctl (const char *path, const char *value);
+
+gboolean nm_utils_get_proc_sys_net_value (const char *path,
+                                          const char *iface,
+                                          guint32 *out_value);
+
+#endif /* NETWORK_MANAGER_UTILS_H */
